@@ -1,8 +1,8 @@
 # application.py: Application class to store properties of the application
 #                 being created
-# $Id: application.py,v 1.43 2004/11/04 22:14:13 agriggio Exp $
+# $Id: application.py,v 1.52 2005/10/14 12:18:31 agriggio Exp $
 # 
-# Copyright (c) 2002-2004 Alberto Griggio <agriggio@users.sourceforge.net>
+# Copyright (c) 2002-2005 Alberto Griggio <agriggio@users.sourceforge.net>
 # License: MIT (see license.txt)
 # THIS PROGRAM COMES WITH NO WARRANTY
 
@@ -55,7 +55,7 @@ class FileDirDialog:
 ##                 self.dir_dialog.SetPath(self.prev_dir)
 ##             dialog = self.dir_dialog
             self.value = misc.DirSelector(self.dir_message,
-                                          flags=self.dir_style)
+                                          style=self.dir_style)
 ##         ok = dialog.ShowModal()
 ##         if ok == wxID_OK:
 ##             self.prev_dir = dialog.GetPath()
@@ -94,9 +94,12 @@ class Application(object):
     def __init__(self, property_window):
         self.property_window = property_window
         self.notebook = wxNotebook(self.property_window, -1)
-        nb_sizer = wxNotebookSizer(self.notebook)
+        if not misc.check_wx_version(2, 5, 2):
+            nb_sizer = wxNotebookSizer(self.notebook)
+            self.notebook.sizer = nb_sizer
+        else:
+            self.notebook.sizer = None
         self.notebook.SetAutoLayout(True)
-        self.notebook.sizer = nb_sizer
         self.notebook.Hide()
         #panel = wxPanel(self.notebook, -1)
         panel = wxScrolledWindow(self.notebook, -1, style=wxTAB_TRAVERSAL)
@@ -105,7 +108,7 @@ class Application(object):
         self.__filename = None # name of the output xml file
         self.klass = "MyApp"
         self.codegen_opt = 0 # if != 0, generates a separate file
-                             # for each class 
+                             # for each class
         def set_codegen_opt(value):
             try: opt = int(value)
             except ValueError: pass
@@ -115,6 +118,9 @@ class Application(object):
         def set_output_path(value): self.output_path = value
         self.use_gettext = False
         def set_use_gettext(value): self.use_gettext = bool(int(value))
+        self.for_version = wxVERSION_STRING[:3]
+        def set_for_version(value):
+            self.for_version = self.for_version_prop.get_str_value()
         self.access_functions = {
             'name': (lambda : self.name, self.set_name),
             'class': (lambda : self.klass, self.set_klass), 
@@ -122,7 +128,8 @@ class Application(object):
             'output_path': (lambda : self.output_path, set_output_path),
             'language': (self.get_language, self.set_language),
             'encoding': (self.get_encoding, self.set_encoding),
-            'use_gettext': (lambda : self.use_gettext, set_use_gettext)
+            'use_gettext': (lambda : self.use_gettext, set_use_gettext),
+            'for_version': (lambda : self.for_version, set_for_version),
             }
         self.use_gettext_prop = CheckBoxProperty(self, "use_gettext", panel,
                                                  "Enable gettext support")
@@ -151,16 +158,22 @@ class Application(object):
                                "Select output file", "Select output directory",
                                wxSAVE|wxOVERWRITE_PROMPT)
 
-        # this "columns-stuff" is to fix a bug (of at least wxGTK)
+##         # this "columns-stuff" is to fix a bug (of at least wxGTK)
         _writers = common.code_writers.keys()
-        if not len(_writers) % 3: columns = 3
-        elif not len(_writers) % 2: columns = 2
-        else: columns = 1
-        
+##         if not len(_writers) % 3: columns = 3
+##         elif not len(_writers) % 2: columns = 2
+##         else: columns = 1
+        columns = 3
+
         self.codewriters_prop = RadioProperty(self, "language", panel,
                                               _writers, columns=columns)
 
         self.codewriters_prop.set_str_value('python')
+
+        self.for_version_prop = RadioProperty(self, "for_version", panel,
+                                              ['2.4', '2.5', '2.6'], columns=3,
+                                              label="wxWidgets compatibility")
+        self.for_version_prop.set_str_value(self.for_version)
         
         # ALB 2004-01-18
         self.access_functions['use_new_namespace'] = (
@@ -197,6 +210,7 @@ class Application(object):
         sizer.Add(szr, 0, wxEXPAND)
         sizer.Add(self.codegen_prop.panel, 0, wxALL|wxEXPAND, 4)
         sizer.Add(self.codewriters_prop.panel, 0, wxALL|wxEXPAND, 4)
+        sizer.Add(self.for_version_prop.panel, 0, wxALL|wxEXPAND, 4)
         sizer.Add(self.use_new_namespace_prop.panel, 0, wxEXPAND)
         sizer.Add(self.overwrite_prop.panel, 0, wxEXPAND)
         sizer.Add(self.outpath_prop.panel, 0, wxEXPAND)
@@ -224,7 +238,7 @@ class Application(object):
             self.name_prop.set_value(self.name)
         else:
             self.name = value
-    set_name_pattern = re.compile('^[a-zA-Z]+[\w0-9]*$')
+    set_name_pattern = re.compile('^[a-zA-Z]+[\w0-9-]*$')
 
     def set_klass(self, value):
         value = "%s" % value
@@ -232,7 +246,7 @@ class Application(object):
             self.klass_prop.set_value(self.klass)
         else:
             self.klass = value
-    set_klass_pattern = re.compile('^[a-zA-Z]+[\w:.0-9]*$')
+    set_klass_pattern = re.compile('^[a-zA-Z]+[\w:.0-9-]*$')
 
     def _get_default_encoding(self):
         """\
@@ -474,7 +488,7 @@ class Application(object):
             if issubclass(FrameClass, wxMDIChildFrame):
                 frame = wxMDIParentFrame(None, -1, '')
                 child = FrameClass(frame, -1, '')
-                child.SetTitle('<Preview> ' + child.GetTitle())
+                child.SetTitle('<Preview> - ' + child.GetTitle())
                 w, h = child.GetSize()
                 frame.SetClientSize((w+20, h+20))
             elif not (issubclass(FrameClass, wxFrame) or
@@ -500,7 +514,7 @@ class Application(object):
                 widget.preview_widget = None
                 widget.preview_button.SetLabel('Preview')
             EVT_CLOSE(frame, on_close)
-            frame.SetTitle('<Preview> %s' % frame.GetTitle())
+            frame.SetTitle('<Preview> - %s' % frame.GetTitle())
             # raise the frame
             frame.Center()
             frame.Show()
