@@ -1,7 +1,7 @@
 # cpp_codegen.py: C++ code generator
-# $Id: cpp_codegen.py,v 1.35 2004/10/21 17:42:54 agriggio Exp $
+# $Id: cpp_codegen.py,v 1.44 2006/02/12 13:01:43 agriggio Exp $
 #
-# Copyright (c) 2002-2004 Alberto Griggio <agriggio@users.sourceforge.net>
+# Copyright (c) 2002-2005 Alberto Griggio <agriggio@users.sourceforge.net>
 # License: MIT (see license.txt)
 # THIS PROGRAM COMES WITH NO WARRANTY
 
@@ -49,6 +49,10 @@ output_name = None
 out_dir = None
 
 
+# ALB 2004-12-05: wx version we are generating code for
+for_version = (2, 4)
+
+
 class ClassLines:
     """\
     Stores the lines of python code for a custom class
@@ -71,6 +75,9 @@ class ClassLines:
         self.done = False # if True, the code for this class has already
                           # been generated
 
+        # ALB 2004-12-08
+        self.event_handlers = [] # lines to bind events 
+
 # end of class ClassLines
 
 
@@ -87,6 +94,13 @@ class SourceFileContent:
         self.classes = {} # classes declared in the file
         self.new_classes = [] # new classes to add to the file (they are
                               # inserted BEFORE the old ones)
+
+        # ALB 2004-12-08
+        self.event_handlers = {} # list of event handlers for each class
+        self.event_table_decl = {}
+        self.event_table_def = {}
+        self.end_class_re = re.compile('^\s*};\s*//\s+wxGlade:\s+end class\s*$')
+
         if classes is None: self.classes = {}
         self.build_untouched_content()
 
@@ -113,6 +127,20 @@ class SourceFileContent:
         block_start = re.compile(r'^\s*//\s*begin\s+wxGlade:\s*'
                                  '(\w*)::(\w+)\s*$')
         block_end = re.compile(r'^\s*//\s*end\s+wxGlade\s*$')
+        # regexp to match event handlers
+        # ALB 2004-12-08
+        event_handler = re.compile(r'^\s*void\s+([A-Za-z_]+\w*)\s*'
+                                   '\([A-Za-z_:0-9]+\s*&\s*\w*\)\s*;\s*'
+                                   '//\s*wxGlade:\s*<event_handler>\s*$')
+        decl_event_table = re.compile(r'^\s*DECLARE_EVENT_TABLE\s*\(\s*\)'
+                                      '\s*;?\s*$')
+        def_event_table = re.compile(r'^\s*BEGIN_EVENT_TABLE\s*\(\s*(\w+)\s*,'
+                                     '\s*(\w+)\s*\)\s*$')
+        event_handlers_marker = re.compile(r'^\s*//\s*wxGlade:\s*add\s+'
+                                           '((?:\w|:)+)\s+event handlers\s*$')
+        prev_was_handler = False
+        events_tag_added = False
+
         inside_block = False
         inside_comment = False
         tmp_in = open(filename)
@@ -147,7 +175,49 @@ class SourceFileContent:
                     inside_block = True
                     out_lines.append('<%swxGlade replace %s %s>' % \
                                      (nonce, result.group(1), result.group(2)))
-                else: out_lines.append(line)
+                else:
+                    dont_append = False
+                    
+                    # ALB 2004-12-08 event handling support...
+                    if is_header and not inside_comment:
+                        result = event_handler.match(line)
+                        if result is not None:
+                            prev_was_handler = True
+                            which_handler = result.group(1)
+                            which_class = class_name #result.group(2)
+                            self.event_handlers.setdefault(
+                                which_class, {})[which_handler] = 1
+                        else:
+                            if prev_was_handler:
+                                # add extra event handlers here...
+                                out_lines.append('<%swxGlade event_handlers %s>'
+                                                 % (nonce, class_name))
+                                prev_was_handler = False
+                                events_tag_added = True
+                            elif not events_tag_added and \
+                                     self.is_end_of_class(line):
+                                out_lines.append('<%swxGlade event_handlers %s>'
+                                                 % (nonce, class_name))
+                            # now try to see if we already have a
+                            # DECLARE_EVENT_TABLE
+                            result = decl_event_table.match(line)
+                            if result is not None:
+                                self.event_table_decl[class_name] = True
+                    elif not inside_comment:
+                        result = event_handlers_marker.match(line)
+                        if result is not None:
+                            out_lines.append('<%swxGlade add %s event '
+                                             'handlers>' % \
+                                             (nonce, result.group(1)))
+                            dont_append = True
+                        result = def_event_table.match(line)
+                        if result is not None:
+                            which_class = result.group(1)
+                            self.event_table_def[which_class] = True
+                    # ----------------------------------------
+                    
+                    if not dont_append:
+                        out_lines.append(line)
             else:
                 # ignore all the lines inside a wxGlade block
                 if block_end.match(line) is not None:
@@ -162,6 +232,10 @@ class SourceFileContent:
         if is_header: self.header_content = "".join(out_lines)
         else: self.source_content = "".join(out_lines)
         
+    def is_end_of_class(self, line):
+        # not really, but for wxglade-generated code it should work...
+        return self.end_class_re.match(line) is not None #[:2] == '};'
+
 # end of class SourceFileContent
 
 # if not None, it is an instance of SourceFileContent that keeps info about
@@ -207,9 +281,11 @@ def initialize(app_attrs):
     multi_files = app_attrs['option']
     
     global classes, header_lines, multiple_files, previous_source, nonce, \
-           _use_gettext, _overwrite
+           _use_gettext, _overwrite, _last_generated_id
     import time, random
 
+    _last_generated_id = 1000
+
     try: _use_gettext = int(app_attrs['use_gettext'])
     except (KeyError, ValueError): _use_gettext = False
 
@@ -217,6 +293,17 @@ def initialize(app_attrs):
     try: _overwrite = int(app_attrs['overwrite'])
     except (KeyError, ValueError): _overwrite = False
 
+    # ALB 2004-12-05
+    global for_version
+    try:
+        for_version = tuple([int(t) for t in
+                             app_attrs['for_version'].split('.')[:2]])
+    except (KeyError, ValueError):
+        if common.app_tree is not None:
+            for_version = common.app_tree.app.for_version
+        else:
+            for_version = (2, 4) # default...
+
     # this is to be more sure to replace the right tags
     nonce = '%s%s' % (str(time.time()).replace('.', ''),
                       random.randrange(10**6, 10**7))
@@ -314,6 +401,17 @@ def finalize():
             comment = '// content of this block not found: ' \
                       'did you rename this class?\n'
             source_content = source_content.replace(tag[0], comment)
+
+        # ALB 2004-12-08
+        tags = re.findall('<%swxGlade event_handlers \w+>' % nonce,
+                          header_content)
+        for tag in tags:
+            header_content = header_content.replace(tag, "")
+        tags = re.findall('<%swxGlade add \w+ event_handlers>' % nonce,
+                          source_content)
+        for tag in tags:
+            source_content = source_content.replace(tag, "")
+        
         # write the new file contents to disk
         common.save_file(previous_source.name + '.h', header_content,
                          'codegen')
@@ -382,6 +480,16 @@ def add_object(top_obj, sub_obj):
                 klass.parents_init.extend(init)
             else: klass.init.extend(init)
             # ---------------------------------------------------------------
+            # -- ALB 2004-12-08 ---------------------------------------------
+            if hasattr(builder, 'get_events'):
+                klass.event_handlers.extend(builder.get_events(sub_obj))
+            elif 'events' in sub_obj.properties:
+                id_name, id = generate_code_id(sub_obj)
+                #if id == '-1': id = 'self.%s.GetId()' % sub_obj.name
+                for event, handler in sub_obj.properties['events'].iteritems():
+                    klass.event_handlers.append((id, event, handler))
+            # ---------------------------------------------------------------
+
             klass.ids.extend(ids)
             if sub_obj.klass != 'spacer':
                 # attribute is a special property which control whether
@@ -400,7 +508,7 @@ def add_object(top_obj, sub_obj):
         klass.layout.extend(layout)
         if multiple_files and \
                (sub_obj.is_toplevel and sub_obj.base != sub_obj.klass):
-            print top_obj.name, sub_obj.name
+            #print top_obj.name, sub_obj.name
             klass.dependencies.append(sub_obj.klass)
         else:
 ##             headers = _obj_headers.get(sub_obj.base, [])
@@ -418,7 +526,7 @@ def add_sizeritem(toplevel, sizer, obj, option, flag, border):
     try: klass = classes[toplevel.klass]
     except KeyError: klass = classes[toplevel.klass] = ClassLines()
     name = obj.name
-    if obj.base == 'wxNotebook':
+    if obj.base == 'wxNotebook' and for_version < (2, 5):
         name = 'new wxNotebookSizer(%s)' % obj.name
     buffer = '%s->Add(%s, %s, %s, %s);\n' % \
              (sizer.name, name, option, flag, border)
@@ -441,14 +549,16 @@ def add_class(code_obj):
         # create a SourceFileContent instance
         filename = os.path.join(out_dir,
                                 code_obj.klass.replace('::', '_') + '.h')
-        if _overwrite or not os.path.exists(filename): prev_src = None
-        else: prev_src = SourceFileContent(os.path.join(out_dir,
-                                                        code_obj.klass))
+        if _overwrite or not os.path.exists(filename):
+            prev_src = None
+        else:
+            prev_src = SourceFileContent(os.path.join(out_dir, code_obj.klass))
 
     if prev_src is not None and prev_src.classes.has_key(code_obj.klass):
         # this class wasn't in the previous version of the source (if any)
         is_new = False 
-    else: is_new = True
+    else:
+        is_new = True
 
     header_buffer = []
     source_buffer = []
@@ -493,6 +603,12 @@ def add_class(code_obj):
     sign_decl1 = ', '.join(tmp_sign)
     sign_inst = ', '.join([ t[1] for t in sign])
 
+
+    # ALB 2004-12-08 event handling
+    event_handlers = classes[code_obj.klass].event_handlers
+    if hasattr(builder, 'get_events'):
+        event_handlers.extend(builder.get_events(code_obj))
+
     if is_new:
         # header file
         hwrite('\nclass %s: public %s {\n' % (code_obj.klass, code_obj.base))
@@ -526,7 +642,26 @@ def add_class(code_obj):
         for o_type, o_name in classes[code_obj.klass].sub_objs:
             hwrite(tabs(1) + '%s* %s;\n' % (o_type, o_name))
         hwrite(tabs(1) + '// end wxGlade\n')
-        hwrite('};\n\n')
+
+        # ALB 2004-12-08 event handling
+        if event_handlers:
+            t = tabs(1)
+            hwrite('\n' + t + 'DECLARE_EVENT_TABLE();\n')
+            hwrite('\npublic:\n')
+            already_there = {}
+            for tpl in event_handlers:
+                if len(tpl) == 4:
+                    win_id, event, handler, evt_type = tpl
+                else:
+                    win_id, event, handler = tpl
+                    evt_type = 'wxCommandEvent'
+                if handler not in already_there:
+                    hwrite(t + 'void %s(%s &event); '
+                           '// wxGlade: <event_handler>\n' %
+                           (handler, evt_type))
+                    already_there[handler] = 1
+        
+        hwrite('}; // wxGlade: end class\n\n')
         
     elif prev_src is not None:
         hwrite(tabs(1) + '// begin wxGlade: %s::ids\n' % code_obj.klass)
@@ -577,8 +712,35 @@ def add_class(code_obj):
         else:
             prev_src.header_content = prev_src.header_content.\
                                       replace(tag, "".join(header_buffer))
-        
 
+        header_buffer = []
+        hwrite = header_buffer.append
+        # ALB 2004-12-08 event handling
+        if event_handlers:
+            already_there = prev_src.event_handlers.get(code_obj.klass, {})
+            t = tabs(1)
+            for tpl in event_handlers:
+                if len(tpl) == 4:
+                    win_id, event, handler, evt_type = tpl
+                else:
+                    win_id, event, handler = tpl
+                    evt_type = 'wxCommandEvent'
+                if handler not in already_there:
+                    hwrite(t + 'void %s(%s &event); // wxGlade: '
+                           '<event_handler>\n' % (handler, evt_type))
+                    already_there[handler] = 1
+            if code_obj.klass not in prev_src.event_table_def:
+                hwrite('\nprotected:\n')
+                hwrite(tabs(1) + 'DECLARE_EVENT_TABLE()\n')
+        tag = '<%swxGlade event_handlers %s>' % (nonce, code_obj.klass)
+        if prev_src.header_content.find(tag) < 0:
+            # no attributes tag found, issue a warning and do nothing
+            print >> sys.stderr, "WARNING: wxGlade events block " \
+                  "not found, event table code NOT generated"
+        else:
+            prev_src.header_content = prev_src.header_content.\
+                                      replace(tag, "".join(header_buffer))
+        
     # source file
     # set the window's style
     prop = code_obj.properties
@@ -629,6 +791,78 @@ def add_class(code_obj):
                                       replace(tag, "".join(source_buffer))
         source_buffer = []
         swrite = source_buffer.append
+
+    # ALB 2004-12-08 event handling code
+    if event_handlers:
+        # 1) event table declaration/definition...
+        if prev_src is not None and \
+               code_obj.klass in prev_src.event_table_decl:
+            has_event_table = True
+        else:
+            has_event_table = False
+        if is_new or not has_event_table:
+            swrite('\nBEGIN_EVENT_TABLE(%s, %s)\n' % \
+                   (code_obj.klass, code_obj.base))
+            
+        swrite(tab + '// begin wxGlade: %s::event_table\n' % code_obj.klass)
+        for tpl in event_handlers:
+            win_id, event, handler = tpl[:3]
+            swrite(tab + '%s(%s, %s::%s)\n' % \
+                   (event, win_id, code_obj.klass, handler))
+        swrite(tab + '// end wxGlade\n')
+        
+        if is_new or not has_event_table:
+            swrite('END_EVENT_TABLE();\n')
+
+        if prev_src is not None and not is_new:
+            tag = '<%swxGlade replace %s event_table>' % (nonce, code_obj.klass)
+            if prev_src.source_content.find(tag) < 0:
+                # no constructor tag found, issue a warning and do nothing
+                print >> sys.stderr, "WARNING: wxGlade %s::event_table block " \
+                      "not found, relative code NOT generated" % \
+                      (code_obj.klass)
+            else:
+                prev_src.source_content = prev_src.source_content.\
+                                          replace(tag, "".join(source_buffer))
+            source_buffer = []
+            swrite = source_buffer.append
+
+        # 2) event handler stubs...
+        if prev_src is not None:
+            already_there = prev_src.event_handlers.get(code_obj.klass, {})
+        else:
+            already_there = {}
+        for tpl in event_handlers:
+            if len(tpl) == 4:
+                win_id, event, handler, evt_type = tpl
+            else:
+                win_id, event, handler = tpl
+                evt_type = 'wxCommandEvent'
+            if handler not in already_there:
+                swrite('\n\nvoid %s::%s(%s &event)\n{\n' % \
+                       (code_obj.klass, handler, evt_type))
+                swrite(tab + 'event.Skip();\n')
+                swrite('}\n')
+                already_there[handler] = 1
+        if is_new or prev_src is None:
+            swrite('\n\n')
+        swrite('// wxGlade: add %s event handlers\n' % code_obj.klass)
+        if is_new or prev_src is None:
+            swrite('\n')
+
+        if prev_src is not None and not is_new:
+            tag = '<%swxGlade add %s event handlers>' % \
+                  (nonce, code_obj.klass)
+            if prev_src.source_content.find(tag) < 0:
+                # no constructor tag found, issue a warning and do nothing
+                print >> sys.stderr, "WARNING: wxGlade %s event handlers " \
+                      "marker not found, relative code NOT generated" % \
+                      (code_obj.klass)
+            else:
+                prev_src.source_content = prev_src.source_content.\
+                                          replace(tag, "".join(source_buffer))
+            source_buffer = []
+            swrite = source_buffer.append
     
 
     # set_properties
@@ -716,12 +950,16 @@ def add_class(code_obj):
         return
 
     if multiple_files:
+        if code_obj.base in obj_builders:
+            classes[code_obj.klass].dependencies.extend(
+                getattr(obj_builders[code_obj.base], 'extra_headers', []))
         if prev_src is not None:
             tag = '<%swxGlade insert new_classes>' % nonce
             prev_src.header_content = prev_src.header_content.replace(tag, "")
             
             # insert the module dependencies of this class
             extra_modules = classes[code_obj.klass].dependencies
+            #print 'extra_modules:', extra_modules, code_obj.base
             deps = ['// begin wxGlade: ::dependencies\n']
             for module in _unique(extra_modules):
                 if module and ('"' != module[0] != '<'):
@@ -755,8 +993,8 @@ def add_class(code_obj):
         hn = os.path.basename(header_file).upper().replace('.', '_')
         hwrite('\n#ifndef %s\n#define %s\n' % (hn, hn))
         # write the module dependecies for this class
-        extra_headers = classes[code_obj.klass].dependencies
-        hwrite('\n// begin wxGlade: dependencies\n')
+        #extra_headers = classes[code_obj.klass].dependencies
+        hwrite('\n// begin wxGlade: ::dependencies\n')
         extra_modules = classes[code_obj.klass].dependencies
         for module in _unique(extra_modules):
             if module and ('"' != module[0] != '<'):
@@ -823,7 +1061,7 @@ def add_app(app_attrs, top_win_class):
     append('IMPLEMENT_APP(%s)\n\n' % klass)
     append('bool %s::OnInit()\n{\n' % klass)
     append(tab + 'wxInitAllImageHandlers();\n') # we add this to avoid troubles
-    append(tab + '%s* %s = new %s(0, -1, "");\n' % \
+    append(tab + '%s* %s = new %s(0, -1, wxT(""));\n' % \
            (top_win_class, top_win, top_win_class))
     append(tab + 'SetTopWindow(%s);\n' % top_win)
     append(tab + '%s->Show();\n' % top_win)
@@ -853,11 +1091,15 @@ def generate_code_size(obj):
     else: name1 = '%s->' % obj.name; name2 = obj.name
     size = obj.properties.get('size', '').strip()
     use_dialog_units = (size[-1] == 'd')
+    if for_version < (2, 5) or obj.parent is None:
+        method = 'SetSize'
+    else:
+        method = 'SetMinSize'
     if use_dialog_units:
-        return name1 + 'SetSize(wxDLG_UNIT(%s, wxSize(%s)));\n' % \
+        return name1 + method + '(wxDLG_UNIT(%s, wxSize(%s)));\n' % \
                (name2, size[:-1])
     else:
-        return name1 + 'SetSize(wxSize(%s));\n' % size
+        return name1 + method + '(wxSize(%s));\n' % size
 
 
 def _string_to_colour(s):
@@ -875,7 +1117,7 @@ def generate_code_foreground(obj):
         color = 'wxColour(%s)' % \
                 _string_to_colour(obj.properties['foreground'])
     except (IndexError, ValueError): # the color is from system settings
-        color = 'wxSystemSettings::GetSystemColour(%s)' % \
+        color = 'wxSystemSettings::GetColour(%s)' % \
                 obj.properties['foreground']
     return intro + 'SetForegroundColour(%s);\n' % color
 
@@ -891,7 +1133,7 @@ def generate_code_background(obj):
         color = 'wxColour(%s)' % \
                 _string_to_colour(obj.properties['background'])
     except (IndexError, ValueError): # the color is from system settings
-        color = 'wxSystemSettings::GetSystemColour(%s)' % \
+        color = 'wxSystemSettings::GetColour(%s)' % \
                 obj.properties['background']
     return intro + 'SetBackgroundColour(%s);\n' % color
 
@@ -907,7 +1149,7 @@ def generate_code_font(obj):
     face = '"%s"' % font['face'].replace('"', r'\"')
     if obj.is_toplevel: intro = ''
     else: intro = '%s->' % obj.name
-    return intro + 'SetFont(wxFont(%s, %s, %s, %s, %s, %s));\n' % \
+    return intro + 'SetFont(wxFont(%s, %s, %s, %s, %s, wxT(%s)));\n' % \
            (size, family, style, weight, underlined, face)
 
 
@@ -931,7 +1173,7 @@ def generate_code_id(obj, id=None):
     else: return '', tokens[0] # we assume name is declared elsewhere
     if not name: return '', val
     if val.strip() == '?':
-        val = str(_last_generated_id)
+        val = 'wxID_HIGHEST + ' + str(_last_generated_id)
         _last_generated_id += 1        
     return '%s = %s' % (name, val), name
 
@@ -1045,8 +1287,38 @@ class DummyPropertyHandler:
 # end of class DummyPropertyHandler
 
 
+class EventsPropertyHandler(object):
+    def __init__(self):
+        self.handlers = {}
+        self.event_name = None
+        self.curr_handler = []
+        
+    def start_elem(self, name, attrs):
+        if name == 'handler':
+            self.event_name = attrs['event']
+
+    def end_elem(self, name, code_obj):
+        if name == 'handler':
+            if self.event_name and self.curr_handler:
+                self.handlers[self.event_name] = ''.join(self.curr_handler)
+            self.event_name = None
+            self.curr_handler = []
+        elif name == 'events':
+            code_obj.properties['events'] = self.handlers
+            return True
+
+    def char_data(self, data):
+        data = data.strip()
+        if data:
+            self.curr_handler.append(data)
+
+# end of class EventsPropertyHandler
+    
+
 # dictionary whose items are custom handlers for widget properties
-_global_property_writers = { 'font': FontPropertyHandler }
+_global_property_writers = { 'font': FontPropertyHandler,
+                             'events': EventsPropertyHandler,
+                             }
 
 # dictionary of dictionaries of property handlers specific for a widget
 # the keys are the class names of the widgets
@@ -1147,3 +1419,17 @@ def _unique(sequence):
     tmp = {}
     for item in sequence: tmp[item] = 1
     return tmp.keys()
+
+
+def get_events_with_type(obj, evt_type):
+    """\
+    Returns the list of event handlers defined for `obj', setting the type of
+    the argument of the handlers (i.e. the event parameter) to `evt_type'
+    """
+    ret = []
+    if 'events' not in obj.properties:
+        return ret
+    id_name, id = generate_code_id(obj)
+    for event, handler in obj.properties['events'].iteritems():
+        ret.append((id, event, handler, evt_type))
+    return ret
