@@ -1,7 +1,7 @@
 # edit_windows.py: base classes for windows used by wxGlade
-# $Id: edit_windows.py,v 1.72 2004/11/06 12:06:11 agriggio Exp $
+# $Id: edit_windows.py,v 1.83 2005/12/28 00:22:02 agriggio Exp $
 # 
-# Copyright (c) 2002-2004 Alberto Griggio <agriggio@users.sourceforge.net>
+# Copyright (c) 2002-2005 Alberto Griggio <agriggio@users.sourceforge.net>
 # License: MIT (see license.txt)
 # THIS PROGRAM COMES WITH NO WARRANTY
 
@@ -11,8 +11,11 @@ from tree import Tree, WidgetTree
 import math, misc, common, sys, config
 import re
 
+# ALB 2004-12-05: event handling support
+from events_mixin import EventsMixin
 
-class EditBase:
+
+class EditBase(EventsMixin):
     """\
     Base class of every window available in the builder.
     """
@@ -73,6 +76,9 @@ class EditBase:
             property_window.SetSize((250, 340))
             property_window.Show(True)
 
+        # ALB 2004-12-05
+        EventsMixin.__init__(self)
+
     def show_widget(self, yes):
         if yes and self.widget is None:
             self.create_widget()
@@ -105,7 +111,7 @@ class EditBase:
             nb_szr = self.notebook.sizer
             self.notebook.DeleteAllPages()
             self.notebook.Destroy()
-            nb_szr.Destroy()
+            if nb_szr is not None: nb_szr.Destroy()
         # ...finally, destroy our widget (if needed)
         if self.widget and not self._dont_destroy:
             self.widget.Destroy()
@@ -117,9 +123,12 @@ class EditBase:
         """
         self.notebook = wxNotebook(self.property_window, -1)
 
-        nb_sizer = wxNotebookSizer(self.notebook)
+        if not misc.check_wx_version(2, 5, 2):
+            nb_sizer = wxNotebookSizer(self.notebook)
+            self.notebook.sizer = nb_sizer
+        else:
+            self.notebook.sizer = None
         self.notebook.SetAutoLayout(True)
-        self.notebook.sizer = nb_sizer
         self.notebook.Hide()
 
         self._common_panel = panel = wxScrolledWindow(self.notebook, -1,
@@ -150,7 +159,7 @@ class EditBase:
             try: common.app_tree.refresh_name(self.node, oldname) #, self.name)
             except AttributeError: pass
             self.property_window.SetTitle('Properties - <%s>' % self.name)
-    set_name_pattern = re.compile('^[a-zA-Z]+[\w0-9]*$')
+    set_name_pattern = re.compile(r'^[a-zA-Z_]+[\w-]*(\[\w*\])*$')
 
     def set_klass(self, value):
         value = "%s" % value
@@ -160,7 +169,7 @@ class EditBase:
             self.klass = value
             try: common.app_tree.refresh_name(self.node) #, self.name)
             except AttributeError: pass
-    set_klass_pattern = re.compile('^[a-zA-Z]+[\w:.0-9]*$')
+    set_klass_pattern = re.compile('^[a-zA-Z_]+[\w:.0-9-]*$')
 
     def popup_menu(self, event):
         if self.widget:
@@ -189,28 +198,72 @@ class EditBase:
         """\
         Updates property_window to display the properties of self
         """
+
+        # Begin Marcello 13 oct. 2005
+        if self.klass == 'wxPanel': # am I a wxPanel under a wxNotebook?
+            if self.parent and self.parent.klass == 'wxNotebook':
+                #pdb.set_trace()
+                nb = self.parent
+                if nb.widget:
+                    i = 0
+                    for tn,ep in nb.tabs: # tn=tabname, ep = editpanel 
+                        if self.name == ep.name:      # If I am under this tab...
+                            nb.widget.SetSelection(i) # ...Show that tab.
+                        i = i + 1
+        if self.parent and self.parent.klass == 'wxPanel': # am I a widget under a wxPanel under a wxNotebook?
+            if self.parent.parent and self.parent.parent.klass == 'wxNotebook':
+                #pdb.set_trace()
+                nb = self.parent.parent
+                if nb.widget:
+                    i = 0
+                    for tn,ep in nb.tabs: # tn=tabname, ep = editpanel 
+                        if self.parent.name == ep.name:
+                            nb.widget.SetSelection(i)
+                        i = i + 1
+        # End Marcello 13 oct. 2005
+
+
         if not self.is_visible(): return # don't do anything if self is hidden
         # create the notebook the first time the function is called: this
         # allows us to create only the notebooks we really need
-        if self.notebook is None: self.create_properties()
+        if self.notebook is None:
+            self.create_properties()
+            # ALB 2004-12-05
+            self.create_events_property()
         sizer_tmp = self.property_window.GetSizer()
-        sizer_tmp = wxPyTypeCast(sizer_tmp, "wxBoxSizer")
-        child = wxPyTypeCast(sizer_tmp.GetChildren()[0], "wxSizerItem")
+        #sizer_tmp = wxPyTypeCast(sizer_tmp, "wxBoxSizer")
+        #child = wxPyTypeCast(sizer_tmp.GetChildren()[0], "wxSizerItem")
+        child = sizer_tmp.GetChildren()[0]
         w = child.GetWindow()
         if w is self.notebook: return
-        w.Hide()
 
+        try:
+            index = -1
+            title = w.GetPageText(w.GetSelection())
+            for i in range(self.notebook.GetPageCount()):
+                if self.notebook.GetPageText(i) == title:
+                    index = i
+                    break
+        except AttributeError, e:
+            #print e
+            index = -1
+        w.Hide()
+        if 0 <= index < self.notebook.GetPageCount():
+            self.notebook.SetSelection(index)
         self.notebook.Reparent(self.property_window)
         child.SetWindow(self.notebook)
         w.Reparent(misc.hidden_property_panel)
-        
+
+        # ALB moved this before Layout, it seems to be needed for wx2.6...
+        self.notebook.Show()
+
         self.property_window.Layout()
         self.property_window.SetTitle('Properties - <%s>' % self.name)
         try: common.app_tree.select_item(self.node)
         except AttributeError: pass
-        self.notebook.Show()
         self.widget.SetFocus()
         
+        
     def on_set_focus(self, event):
         """\
         Event handler called when a window receives the focus: this in fact is
@@ -227,7 +280,8 @@ class EditBase:
         when loading this object from an xml file. handler must provide
         three methods: 'start_elem', 'end_elem' and 'char_data'
         """
-        return None
+        # ALB 2004-12-05
+        return EventsMixin.get_property_handler(self, prop_name)
 
     def clipboard_copy(self, *args):
         """\
@@ -574,7 +628,8 @@ class WindowBase(EditBase):
                     self.widget.SetMinSize(size)
                 self.widget.SetSize(size)
                 try:
-                    self.sizer.set_item(self.pos, size=self.widget.GetSize())
+                    #self.sizer.set_item(self.pos, size=self.widget.GetSize())
+                    self.sizer.set_item(self.pos, size=size)
                 except AttributeError:
                     pass
 
@@ -603,7 +658,7 @@ class WindowBase(EditBase):
                     self.props[self.index] = str(data.strip())
             # end of class FontHandler
             return FontHandler(self)
-        return None
+        return EditBase.get_property_handler(self, name)
 
     def get_disabled(self):
         return self.disabled_p
@@ -673,18 +728,23 @@ class ManagedBase(WindowBase):
                        'wxADJUST_MINSIZE')
         # ALB 2004-08-16 - see the "wxPython migration guide" for details...
         if misc.check_wx_version(2, 5, 2):
-            self.flag = wxFIXED_MINSIZE
+            self.flag = wxADJUST_MINSIZE #wxFIXED_MINSIZE
             self.flags_pos += (wxFIXED_MINSIZE, )
             flag_labels += ('wxFIXED_MINSIZE', )
         sizer.add_item(self, pos)
 
         szprop = self.sizer_properties
-        szprop['option'] = SpinProperty(self, "option", None, 0, (0, 1000))
+        #szprop['option'] = SpinProperty(self, "option", None, 0, (0, 1000))
+        from layout_option_property import LayoutOptionProperty, \
+             LayoutPosProperty
+        szprop['option'] = LayoutOptionProperty(self, sizer)
+        
         szprop['flag'] = CheckListProperty(self, 'flag', None, flag_labels)
         szprop['border'] = SpinProperty(self, 'border', None, 0, (0, 1000))
-        pos_p = szprop['pos'] = SpinProperty(self, 'pos', None, 0, (0, 1000))
-        def write(*args, **kwds): pass
-        pos_p.write = write # no need to save the position
+##         pos_p = szprop['pos'] = SpinProperty(self, 'pos', None, 0, (0, 1000))
+##         def write(*args, **kwds): pass
+##         pos_p.write = write # no need to save the position
+        szprop['pos'] = LayoutPosProperty(self, sizer)
 
     def finish_widget_creation(self, sel_marker_parent=None):
         if sel_marker_parent is None: sel_marker_parent = self.parent.widget
@@ -701,6 +761,8 @@ class ManagedBase(WindowBase):
         szp['flag'].set_value(self.get_flag())
         szp['border'].set_value(self.get_border())
         szp['pos'].set_value(self.pos-1)
+##         if self.properties['size'].is_active():
+##             self.sizer.set_item(self.pos, size=self.widget.GetSize())
 
     def create_properties(self):
         WindowBase.create_properties(self)
@@ -762,7 +824,10 @@ class ManagedBase(WindowBase):
                 w, h = [ int(v) for v in size.split(',') ]
                 if use_dialog_units:
                     w, h = wxDLG_SZE(self.widget, (w, h))
-            else: w, h = self.widget.GetBestSize()
+                if value:
+                    w, h = 1, 1
+            else:
+                w, h = self.widget.GetBestSize()
             self.sizer.set_item(self.pos, option=value, size=(w, h))
         except AttributeError, e:
             print e
@@ -908,12 +973,13 @@ class TopLevelBase(WindowBase, PreviewMixin):
     _is_toplevel = True
     
     def __init__(self, name, klass, parent, id, property_window, show=True,
-                 has_title=True):
+                 has_title=True, title=None):
         WindowBase.__init__(self, name, klass, parent, id, property_window,
                             show=show)
         self.has_title = has_title
         if self.has_title:
-            self.title = self.name
+            if title is None: title = self.name
+            self.title = title
             self.access_functions['title'] = (self.get_title, self.set_title)
             self.properties['title'] = TextProperty(self, 'title', None)
         self.sizer = None # sizer that controls the layout of the children
@@ -924,7 +990,10 @@ class TopLevelBase(WindowBase, PreviewMixin):
         WindowBase.finish_widget_creation(self)
         self.widget.SetMinSize = self.widget.SetSize
         if self.has_title:
-            self.widget.SetTitle(self.properties['title'].get_value())
+            self.widget.SetTitle(misc.design_title(
+                self.properties['title'].get_value()))
+        elif hasattr(self.widget, 'SetTitle'):
+            self.widget.SetTitle(misc.design_title(self.name))
         EVT_LEFT_DOWN(self.widget, self.drop_sizer)
         EVT_ENTER_WINDOW(self.widget, self.on_enter)
         EVT_CLOSE(self.widget, self.hide_widget)
@@ -997,7 +1066,7 @@ class TopLevelBase(WindowBase, PreviewMixin):
     def set_title(self, value):
         self.title = misc.wxstr(value)
         if self.widget:
-            self.widget.SetTitle(value)
+            self.widget.SetTitle(misc.design_title(value))
 
     def set_sizer(self, sizer):
         self.sizer = sizer
